<?php
class ShopsatoConnector
{
    private $url;
    private $token;

    public function __construct($url, $token)
    {
        $this->url = $url;
        $this->token = $token;
    }

    public function getHash()
    {
        return wp_hash(NONCE_SALT . sha1(get_site_url()) . NONCE_KEY, 'nonce');
    }
    public function connect($endpoint, $data = "")
    {
        $headers = [
            'Cache-Control: no-cache',
            'Content-Type: application/x-www-form-urlencoded; charset=utf-8',
            'User-Agent: Mozilla/5.0 (X11; Ubuntu; Linux i686; rv:28.0) Gecko/20100101 Firefox/28.0',
            'X-MicrosoftAjax: Delta=true',
            "X-AUTH-TOKEN: " . $this->token
        ];

        $ch = curl_init($this->url . '/' . $endpoint);


        # Return response instead of printing.
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        if (trim($data) != "") {
            curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json', "X-AUTH-TOKEN: " . $this->token));
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        } else {
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "GET");
            curl_setopt($ch, CURLOPT_PUT, false);
        }

        $result = curl_exec($ch);

        $response = json_decode($result, true);
        if (curl_getinfo($ch, CURLINFO_HTTP_CODE) != 200 && curl_getinfo($ch, CURLINFO_HTTP_CODE) != 201) {
            $response =  printf("Při operaci nastala chyba (HTTP %d): %sn", curl_getinfo($ch, CURLINFO_HTTP_CODE), $response);
            $error_msg = curl_error($ch);

            print_r($error_msg);
        }
        curl_close($ch);
        return $response;
    }

    public function getStatus()
    {
        $response = $this->connect('status');
        if ($response['status'] == 'ok') {
            echo '<h3>Připojen k Shopstato API je funkční.</h3>';
            echo 'Název e-shopu: <strong>' . $response['nameEshop'] . '</strong><br>';
            echo 'Období: <strong>' . $response['startCurrentMonth'] . ' - ' . $response['endCurrentMonth'] . '</strong><br>';
            echo 'Poslední objednávka: kód: <strong>' . $response['lastOrder']['code'] . '</strong> datum vytvoření: <strong>' . date('H:i:s d.m.Y', strtotime($response['lastOrder']['createDate']))  . '</strong> datum aktualizace: <strong>' . date('H:i:s d.m.Y', strtotime($response['lastOrder']['updateDate']))  . '</strong><br>';
            echo 'Obrat: <strong>' . $response['sumOrder'] . '</strong><br>';
            echo 'Počet objednávek: <strong>' . $response['countOrders'] . '</strong><br>';
            echo 'Limit odeslaných objednávek tento měsíc: <strong>' . $response['limitOrder'] . '</strong><br>';
            echo 'Počet odeslaných objednávek tento měsíc: <strong>' . $response['currentCountOrder'] . '</strong><br>';
        } else {
            echo '<h3>Chyba připojení k Shopstato API.</h3>';
            echo '<strong>Zkontrolujte si token a adresu API</strong><br>';
            if ($response['status'] == 'ERROR') {
                echo 'Status: ' . $response['status'] . '<br>';
                echo 'Důvod: ' . $response['message'] . '<br>';
            } else {
                echo 'Status: ' . $response['status'] . '<br>';
                echo 'Detail: ' . $response['detail'] . '<br>';
            }
        }
    }

    public function getOrderData($orderId)
    {
        $orderData = wc_get_order($orderId);
        $order_data = $orderData->get_data();
        $round = strtolower($order_data['currency']) == 'czk' ? 0 : 2;
        $order = [];
        $order['order']['code'] = $order_data['number']; // kód objednávky max. délka 15znaků
        $order['order']['createDate'] = date("Y-m-d H:i:s", strtotime($orderData->get_date_created('date'))); // datum vytvoření objednávky ISO
        $order['order']['updateDate'] = date("Y-m-d H:i:s", strtotime($orderData->get_date_modified('date'))); // datum aktualizace objednávky ISO
        $order['order']['totalPrice'] = round($order_data['total'], $round); // cena celé objednávky s dph
        $order['order']['withoutVat'] = round(($order_data['total'] - $order_data['total_tax']), $round); // cena celé objednávky bez dph 
        $order['order']['currency'] = 1; // 1 ->CZK, 2 ->EURO
        $order['order']['shipping'] = $order_data['shipping_total']; //cena dopravy bez dph  
        $order['order']['statusOrder'] = $order_data['status']; //stav objednávky,max. délka 100znaků, povinný údaj
        $order['order']['weight'] = 0; //váha v kg
        $order['order']['otherId'] = $order_data['id']; // id objednávky 
        $order['order']['shippingName'] = $orderData->get_shipping_method(); // název dopravy,max. délka 100znaků, povinný údaj
        $order['order']['paymentName'] = $order_data['payment_method_title']; // název platby max. délka 100znaků, povinný údaj
        $order['order']['shippingVat'] = 0; // dph dopravy, povinný údaj
        $order['order']['paymentVat'] = 0; // dph platby, povinný údaj
        $order['order']['paid'] =  1; // zda je objednávka zaplacena, pokud ano 1 pokud ne 0
        $order['customer']['ip'] = $order_data['customer_ip_address'];

        switch (strtolower($order_data['billing']['country'])) {
            case "cz":
                $billingState = 1;
                break;
            case "sk":
                $billingState = 2;
                break;
            default:
                $billingState = -1;
        }

        $order['address']['name'] = $order_data['billing']['first_name']; // fakturační jméno max. délka 100znaků
        $order['address']['surname'] = $order_data['billing']['last_name']; // fakturační přijmení max. délka 100znaků
        $order['address']['street'] = $order_data['billing']['address_1'] . ' ' . $order_data['billing']['address_2']; // fakturační ulice max. délka 100znaků, povinný údaj
        $order['address']['town'] = $order_data['billing']['city']; // fakturační město max. délka 100znaků, povinný údaj
        $order['address']['zip'] = $order_data['billing']['postcode']; // fakturační psč max. délka 100znaků, povinný údaj
        $order['address']['country'] = $billingState; // fakturační stát, 1-> CZ, 2-> SK , povinný údaj
        $order['address']['company'] = $order_data['billing']['company']; // fakturační název spolčenosti max. délka 100znaků 
        $order['address']['crn'] = 0; // fakturační ič , 
        $order['address']['taxId'] = 0; // fakturační dič 
        $order['address']['crnVatId'] = 0; //pro SK
        $order['address']['email'] = $order_data['billing']['email']; // fakturační email, povinný údaj
        $order['address']['phone'] = $order_data['billing']['phone']; // fakturační telefon, povinný údaj
        switch (strtolower($order_data['shipping']['country'])) {
            case "cz":
                $deliveryState = 1;
                break;
            case "sk":
                $deliveryState = 2;
                break;
            default:
                $deliveryState = -1;
        }
        $order['address']['deliveryName'] = $order_data['shipping']['first_name']; // doručovací jméno max. délka 100znaků
        $order['address']['deliverySurname'] = $order_data['shipping']['last_name']; // doručovací přijmení max. délka 100znaků
        $order['address']['deliveryStreet'] = $order_data['shipping']['address_1'] . ' ' . $order_data['shipping']['address_2']; // doručovací ulice max. délka 100znaků
        $order['address']['deliveryTown'] = $order_data['shipping']['city']; // doručovací město max. délka 100znaků
        $order['address']['deliveryZip'] = $order_data['shipping']['postcode']; // doručovací psč max. délka 100znaků
        $order['address']['deliveryPhone'] = ''; // doručovací telefon
        $order['address']['deliveryCountry'] = $deliveryState; // doručovací stát 1-> CZ, 2-> SK
        $order['address']['deliveryCompany'] = $order_data['shipping']['company']; // doručovací společnost, max. délka 100znaků
        $order['address']['deliveryCrn'] = ''; // doručovací ič
        $order['address']['deliveryTaxId'] = ''; // doručovací dič
        $order['address']['deliveryCrnVatId'] = ''; //pro SK
        $order['address']['deliveryEmail'] = ''; // doručovací email
        $priceProducts = 0;
        $items = [];
        foreach ($orderData->get_items() as $item_id => $item) {
            $productData = $item->get_data();
            $priceProducts += $productData['total'];
            $product = [];
            $product['productId'] = $productData['product_id']; //id produktu , povinný údaj
            $product['code'] = $productData['product_id']; // kód produktu , povinný údaj
            $product['ean'] = 0; //ean produktu
            $product['name'] = $productData['name']; // název produktu, max. délka 120znaků, povinný údaj
            $product['count'] = $productData['quantity']; // počet kusů, povinný údaj
            $product['price'] =  round($productData['total'], $round); // cena bez dph, povinný údaj
            $product['priceVat'] =  round($productData['total'] + $productData['total_tax'], $round); // cena s dph  , povinný údaj
            $product['vat'] = round((($product['priceVat'] / $product['price']) - 1) * 100, $round); //dph sazba, povinný údaj
            $product['weight'] = 0; // v kg
            $product['producer'] = ''; // vyrobce
            $product['unitText'] = ''; // jednotka
            $product['buyPriceWithVat'] = 0; // nakpní cena s dph
            $product['buyPriceWihoutVat'] = 0; // nakupní cena bez dph
            $product['buyPriceVat'] = 0; // dph sazba
            $product['warranty'] = 0;
            $product['discoutPrice'] = 0;
            $product['discountPercent'] = 0;
            $product['originalPrice'] = 0;
            $items[] = $product;
        }
        $order['order']['payment'] = round(($order['order']['withoutVat'] - $order['order']['shipping'] - $priceProducts), $round); //cena platby bez dph

        $order['order']['withoutVat'] = round(($order['order']['withoutVat'] - $order['order']['shipping'] - $order['order']['payment']), $round);
        $order['items'] = $items;
        return $order;
    }

    public function getStorageDataFromOrder()
    {

        $args = array('post_type' => 'product', 'posts_per_page' => -1);
        $pr = get_posts($args);
        $products = [];
        for ($i = 0; $i < count($pr); $i++) {
            $products[$i] = [];

            $pro = new WC_Product($pr[$i]->ID);
            if (trim($pro->sku) == "") {
                continue;
            }
            $products[$i]['productId'] = $pr[$i]->ID;
            $products[$i]['code'] = $pro->sku;
            $products[$i]['amount'] =$pro->get_stock_quantity();
            $tickets = new WC_Product_Variable($pr[$i]->ID);
            $variables = $tickets->get_available_variations();
            if (count($variables) > 0) {
                for ($v = 0; $v < count($variables); $v++) {
                    if ($variables[$v]['sku'] == $pro->sku || trim($variables[$v]['sku']) == "") {
                        continue;
                    }                 
                    $variation = new WC_Product_variation($variables[$v]['variation_id']);
                    $products[$i . '_' . $v] = [];
                    $products[$i . '_' . $v]['productId'] = $variables[$v]['variation_id'];
                    $products[$i . '_' . $v]['code'] = $variables[$v]['sku'];
                    $products[$i . '_' . $v]['amount'] = $variation->get_stock_quantity();

                }
            }
        }
        return $products;
    }
}
